#define PYGGI_MODULE
#include "pyggi.h"

#define PACK_FUNC(s, c) PyCFunction_New(&Pack_methods[c], (PyObject*)s)


static PyObject *
Pack_unPack(Pack * self) {
	PyObject * r;
	ggi_color * c;
	double fa,fb,fc;
	size_t i;
	
	c=calloc(self->length, sizeof(ggi_color));
	if(!c) {
		PyErr_SetString(PyExc_MemoryError, "out of memory");
		return NULL;
	}
	if (ggiUnpackPixels(self->vis->vis,
			    self->pixels, c, (int)self->length)) {
		free(c);
		PyErr_SetString(PyExc_SystemError,"ggiUnpackPixels failed");
		return NULL;
	}
	
	r=PyList_New((int)self->length);
	for(i=0;i<self->length;i++) {
		color2float(c+i,fa,fb,fc);
		PyList_SET_ITEM(r,i,Py_BuildValue("fff",fa,fb,fc));
	}
	free(c);	
	return r;
}


enum {
	PACK_UNPACK = 0
};

static struct PyMethodDef Pack_methods[] = {
	
	{ "unPack", (PyCFunction)Pack_unPack, METH_NOARGS, (char*) NULL },
	
	/* ---------- end -----------*/
	
	{ NULL, NULL, 0, (char*)NULL }
};

static PyObject*
Pack_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
	Pack *self;
	
	self = (Pack*)type->tp_alloc(type, 0);
	if (self!=NULL) {
		self->vis    = NULL;
		self->pixels = NULL;
		self->length = 0;
		self->alloc = 0;
	}
	return (PyObject*)self;
}

static int
Pack_init(Pack * self, PyObject *args, PyObject *kwds) {
	Visual   * o;
	PyObject * i;
	int length;

	if(!PyArg_ParseTuple(args, "Oi", (PyObject**)(&o), &length)) {
		return -1;
	}
	ASSERT_VISUAL(o, -1);
	
	self->length = length;
	self->alloc  = ((self->length*GT_SIZE(o->mode.graphtype))/8+1);
	self->pixels = malloc(self->alloc);
	if(!self->pixels) {
		PyErr_SetString(PyExc_MemoryError, "out of memory");
		return -1;
	}
	self->vis = o;
	Py_INCREF(self->vis);
       	return 0;
}

static void
Pack_dealloc(Pack *self) {
	if(self->vis) {
		Py_DECREF(self->vis);
	}
	if(self->pixels)
		free(self->pixels);
	self->ob_type->tp_free((PyObject*)self);
}

static int
Pack_setattr(Pack *self, char *name, PyObject *value) {
	PyErr_SetString(PyExc_AttributeError, name);
	return -1;
	
	self=NULL; name=NULL; value=NULL;
}

static PyObject*
Pack_getattr(Pack *self, char *name) {
	switch(name[0]) {
	case 'l':
		if(strcmp(name,"length")==0)
			return PyInt_FromLong((long)self->length);
		break;
	case 'u':
		if(strcmp(name,"unPack")==0)
			return PACK_FUNC(self, PACK_UNPACK);
		break;
	}
	PyErr_SetString(PyExc_AttributeError, name);
	return NULL;
}

static PyObject*
Pack_str(Pack *self) {
	return PyString_FromString("Pack");
}

static PyObject*
Pack_repr(Pack *self) {
	return PyString_FromString("Pack()");
}


PyTypeObject PackType = {
	PyObject_HEAD_INIT(NULL)
	0,                          /* ob_size         */
	"GGI._ggimodule.Pack",      /* tp_name         */
	sizeof(Pack),               /* tp_basicsize    */
	0,                          /* tp_itemsize     */
	(destructor)Pack_dealloc,   /* tp_dealloc      */
	0,                          /* tp_print        */
	(getattrfunc)Pack_getattr,  /* tp_getattr      */
	(setattrfunc)Pack_setattr,  /* tp_setattr      */
	0,                          /* tp_compare      */
	(reprfunc)Pack_repr,        /* tp_repr         */
	0,                          /* tp_as_number    */
	0,                          /* tp_as_sequence  */
	0,                          /* tp_as_mapping   */
	0,                          /* tp_hash         */
	0,                          /* tp_call         */
	(reprfunc)Pack_str,         /* tp_str          */
	0,                          /* tp_getattro     */
	0,                          /* tp_setattro     */
	0,                          /* tp_as_buffer    */
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
	"Pack objects",            /* tp_doc          */
	
	/* 2.0 */
	0,		            /* tp_traverse     */
	0,		            /* tp_clear        */
	/* 2.1 */
	0,		            /* tp_richcompare  */
	0,		            /* tp_weaklistoffset */
	/* 2.2 */
	0,		            /* tp_iter         */
	0,		            /* tp_iternext     */
	0,                          /* tp_methods      */
	0,                          /* tp_members      */
	0,                          /* tp_getset       */
	0,                          /* tp_base         */
	0,                          /* tp_dict         */
	0,                          /* tp_descr_get    */
	0,                          /* tp_descr_set    */
	0,                          /* tp_dictoffset   */
	(initproc)Pack_init,       /* tp_init         */
	0,                          /* tp_alloc        */
	Pack_new,                  /* tp_new          */
	0,                          /* tp_is_gc        */
	0,                          /* tp_bases        */
        0,                          /* tp_mro          */
	0,                          /* tp_cache        */
	0,                          /* tp_subclasses   */
	0,                          /* tp_weaklist     */
	0                           /* tp_del          */
};
